/*
 * FILE:     audio_mix.c
 * PROGRAM:  RAT
 * AUTHOR:   Isidor Kouvelas / Orion Hodson (MMX)
 * MODIFIED: V.J.Hardman + Colin Perkins
 *
 * These functions taken from audio.c revision 1.4, since they caused
 * problems for the mixer if left in that file. Probably we should 
 * move them into mix.[ch] at some stage in the future...
 *
 * $Revision: 1.2 $
 * $Date: 1999/02/10 12:50:25 $
 *
 * Copyright (c) 1995,1996 University College London
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted, for non-commercial use only, provided
 * that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Computer Science
 *      Department at University College London
 * 4. Neither the name of the University nor of the Department may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 * Use of this software for commercial purposes is explicitly forbidden
 * unless prior written permission is obtained from the authors.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "bat_include.h"

void
mix_audio(int16 *v0, int16 *v1, size_t len)
{
	int16 *v0e = v0 + len;
	int tmp;

	assert(v0 != NULL);
	assert(v1 != NULL);

	while(v0 != v0e) {
		tmp = *v0 + *v1++;
		if (tmp>32767) {
			tmp = 32767;
		} else if (tmp<-32768) {
			tmp = -32768;
		}
		*v0++ = tmp;
	}
}

#ifdef WIN32

/* For type mix unit size this function is twice as fast
 * as non-mmx function.  For longer buffers it gets better.
 */

void 
mmx_mix_audio(int16 *v0,  int16 *v1, size_t len)
{
	unsigned int tmp, endq, i;

	/* The end of where we can do quad sample addition */
	endq  = ((len * sizeof(short)) / 4) * 4;

	__asm {
		mov esi, 0
		mov eax, v0
		mov ebx, v1
		jmp START_L1
LOOP_L1:
		add esi, 8
START_L1:
		movq   mm0, [eax + esi]
		paddsw mm0, [ebx + esi]
		movq   [eax + esi], mm0
		cmp	   esi, endq
		jb	   LOOP_L1	
	/* Clear state */
		emms
	}

	/* This is not the most efficient mix but setup for more efficient mix
	 * from here is almost certainly more inefficient.
	 */
	for (i = endq / 2; i < len; i++) {
		tmp = v0[i] + v1[i];
		if (tmp > 32767) {
			tmp = 32767;
		} else if (tmp < -32767) {
			tmp = -32767;
		}
		v0[i] = (short)tmp;
	}
}

#endif /* WIN32 */
